# -*- coding: utf-8 -*-
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from dc17.models.attendee import Attendee


SUBJECT = u'Your accommodation request for DebConf17'
TXT = u"""Hi,

Your registration profile for DebConf17 indicates that you have requested
on-site accommodation for your stay.

If you've already looked into this accommodation arrangement by reading the
information on the wiki [1], and you have no questions or concerns, then no
further action is required on your part.

However, if you've delayed looking into your accommodation details, then please
do so now.

If you find that the on-site arrangement of shared classroom dorms is not
adequate for you, please modify your registration profile and select
alternative accommodation (page 7, "I would like to request alternative
accommodation" checkbox). Doing this will have no effect on your bursary
application.

*After May 31st, we will consider your accommodation choice as final. We will
not be able to satisfy any last-minute re-accommodation requests.*

We want to ensure everyone has a most pleasant stay, so if you have any
question or concerns, please get in touch with us.

The DebConf registration team

[1] https://wiki.debconf.org/wiki/DebConf17/Accommodation#On-site
"""


class Command(BaseCommand):
    help = 'Send an email about making final accomm choices'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something')

    def badger(self, attendee, dry_run):
        if dry_run:
            user_profile = attendee.user.userprofile
            print('I would badger: %s'
                  % user_profile.display_name().encode('utf-8'))
            return
        to = attendee.user.email
        email_message = EmailMultiAlternatives(SUBJECT, TXT, to=[to])
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        for attendee in Attendee.objects.filter(
                accomm__isnull=False,
                user__bursary__request_accommodation=True):
            if not attendee.accomm.nights.exists():
                continue

            self.badger(attendee, dry_run)



# Generated by Django 2.2.28 on 2022-12-23 12:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('minidebconf', '0002_auto_20221009_2000'),
    ]

    operations = [
        migrations.CreateModel(
            name='RegistrationType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=64, verbose_name='Description')),
            ],
        ),
        migrations.AddField(
            model_name='registration',
            name='phone_number',
            field=models.CharField(blank=True, max_length=24, null=True),
        ),
        migrations.AddField(
            model_name='registration',
            name='registration_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='minidebconf.RegistrationType'),
        ),
    ]



# Generated by Django 4.2.8 on 2023-12-23 20:15

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("minidebconf", "0005_registration_arranged_accommodation_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="registration",
            name="notes",
            field=models.TextField(blank=True, null=True, verbose_name="Notes"),
        ),
        migrations.AddField(
            model_name="registration",
            name="travel_cost",
            field=models.IntegerField(
                default=None, null=True, blank=True, verbose_name="Travel costs"
            ),
        ),
        migrations.AddField(
            model_name="registration",
            name="travel_reimbursement",
            field=models.BooleanField(
                default=False, verbose_name="Reimburse travel costs"
            ),
        ),
    ]



from django.contrib import admin
from django.contrib.admin.templatetags.admin_list import _boolean_icon

from register.models.childcare import ChildCare


class ChildCareAdmin(admin.ModelAdmin):
    list_display = ('attendee', 'full_name', 'email', 'reconfirm',)
    list_filter = ('attendee__reconfirm',)
    search_fields = (
        'attendee__user__username', 'attendee__user__first_name',
        'attendee__user__last_name'
    )

    def full_name(self, instance):
        return instance.attendee.user.get_full_name()
    full_name.admin_order_field = 'attendee__user__last_name'

    def email(self, instance):
        return instance.attendee.user.email
    email.admin_order_field = 'attendee__user__email'

    def reconfirm(self, instance):
        return _boolean_icon(instance.attendee.reconfirm)
    reconfirm.short_description = 'Confirmed?'
    reconfirm.admin_order_field = 'attendee__reconfirm'


admin.site.register(ChildCare, ChildCareAdmin)



from django.conf import settings
from django.test import Client
import re


class TestRobots:
    def test_robots(self, db):
        client = Client()
        response = client.get('/robots.txt')
        assert response.status_code == 200
        assert "Disallow:" in response.content.decode("utf-8").splitlines()

    def test_robots_sandbox(self, db, monkeypatch):
        monkeypatch.setattr(settings, "SANDBOX", True)
        client = Client()
        response = client.get('/robots.txt')
        assert response.status_code == 200
        assert "Disallow: /" in response.content.decode("utf-8").splitlines()



import datetime

from django.conf import settings

# A little thumb-sucking going on in here. May need to be more generic...
ARRIVE_ON_OR_AFTER = settings.DEBCONF_DATES[0][1]
ORGA_ARRIVE_ON_OR_AFTER = ARRIVE_ON_OR_AFTER - datetime.timedelta(days=3)
LEAVE_ON_OR_BEFORE = settings.DEBCONF_DATES[-1][2] + datetime.timedelta(days=1)
T_SHIRT_SWAP_ON_OR_AFTER = LEAVE_ON_OR_BEFORE - datetime.timedelta(days=4)


def meals(orga=False):
    day = ARRIVE_ON_OR_AFTER
    while day <= LEAVE_ON_OR_BEFORE:
        if settings.DEBCONF_BREAKFAST:
            yield ('breakfast', day)
        if day < LEAVE_ON_OR_BEFORE:
            yield ('lunch', day)
            yield ('dinner', day)
        day += datetime.timedelta(days=1)


def nights(orga=False):
    day = ARRIVE_ON_OR_AFTER
    if orga:
        day = ORGA_ARRIVE_ON_OR_AFTER
    while day < LEAVE_ON_OR_BEFORE:
        yield day
        day += datetime.timedelta(days=1)


def parse_date(date):
    return datetime.date(*(int(part) for part in date.split('-')))


def meal_choices(orga=False):
    for meal, date in meals(orga=orga):
        date = date.isoformat()
        yield ('{}_{}'.format(meal, date),
               '{} {}'.format(meal.title(), date))


def conference_dinner_meal():
    date = settings.DEBCONF_CONFERENCE_DINNER_DAY
    if not date:
        return None
    return 'dinner_{}'.format(date.isoformat())


def skipped_meals():
    for meal, date in getattr(settings, 'DEBCONF_SKIPPED_MEALS', ()):
        yield f'{meal}_{date.isoformat()}'


def night_choices(orga=False):
    for date in nights(orga=orga):
        date = date.isoformat()
        yield ('night_{}'.format(date), 'Night of {}'.format(date))


def get_ranges_for_dates(dates):
    """Get ranges of consecutive dates for the given set of dates"""
    one = datetime.timedelta(days=1)

    ranges = []

    range_start = None
    range_end = None
    for date in sorted(dates):
        if range_start is None:
            range_start = date
        else:
            if date != range_end + one:
                # dates not consecutive, save old range and start new one
                ranges.append([range_start, range_end])
                range_start = date

        range_end = date

    ranges.append([range_start, range_end])
    return ranges



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-03-09 01:44
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ChildCare',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('needs', models.TextField(blank=True)),
                ('details', models.TextField(blank=True)),
                ('attendee', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='child_care', to='register.Attendee')),
            ],
        ),
        migrations.RemoveField(
            model_name='accomm',
            name='childcare',
        ),
        migrations.RemoveField(
            model_name='accomm',
            name='childcare_details',
        ),
        migrations.RemoveField(
            model_name='accomm',
            name='childcare_needs',
        ),
    ]



from collections import Counter, OrderedDict, defaultdict

from django.conf import settings
from django.core.cache import cache
from django.core.exceptions import ObjectDoesNotExist

from bakery.views import BuildableTemplateView

from bursary.models import Bursary, BURSARY_STATUS_CHOICES
from front_desk.models import CheckIn
from register.models import Attendee, Meal, user_is_registered


def clean_almostdicts(value):
    if not isinstance(value, dict):
        return value

    return OrderedDict(
        (k, clean_almostdicts(v)) for k, v in value.items()
    )


class StatisticsView(BuildableTemplateView):
    build_path = 'register/statistics/index.html'
    template_name = 'register/statistics.html'
    cache_key = 'register:statistics'
    cache_timeout = 30*60 if not settings.DEBUG else 10

    def get_context_data(self, **kwargs):
        from register.views import STEPS
        retval = cache.get(self.cache_key)
        if retval:
            return retval

        attendees = Attendee.objects.all()
        attendees_count = len(attendees)
        attendees_registered = 0
        attendees_approved = 0
        attendees_pending_approval = 0
        attendees_denied_approval = 0
        attendees_confirmed = 0
        attendees_arrived = 0
        attendees_by_country = defaultdict(Counter)
        attendees_by_language = defaultdict(Counter)
        attendees_by_gender = defaultdict(Counter)
        fees = defaultdict(Counter)
        tshirts = defaultdict(Counter)
        tshirts_total = Counter()
        shoes = defaultdict(Counter)
        shoes_total = Counter()
        accomm_total = 0
        accomm_confirmed = 0
        accommodation = defaultdict(Counter)
        food_total = 0
        food_confirmed = 0
        food_restrictions = defaultdict(Counter)
        food_restrictions['No restrictions']  # get that on top of the list
        meals = defaultdict(
            lambda: defaultdict(Counter)
        )
        for attendee in attendees:
            if attendee.completed_register_steps >= len(STEPS) - 1:
                attendees_registered += 1
            else:
                continue

            paid = attendee.paid()

            try:
                bursary = Bursary.objects.get(user=attendee.user)
            except Bursary.DoesNotExist:
                bursary = Bursary()

            approved = attendee.registration_approved()
            if approved:
                attendees_approved += 1
            elif bursary.attendance_status in ('submitted', 'ranked', 'pending'):
                attendees_pending_approval += 1
            elif bursary.attendance_status in ('denied', 'expired', 'canceled'):
                attendees_denied_approval += 1

            checked_in = CheckIn.objects.filter(attendee=attendee).exists()

            if checked_in:
                attendees_arrived += 1

            reconfirm = any((
                checked_in,
                not bursary.request_any and attendee.billable() and paid,
                not bursary.request_any and not attendee.billable()
                    and attendee.final_dates,
                bursary.request_any and bursary.status_in(None, ['accepted']),
                attendee.reconfirm,
            )) and approved

            if reconfirm:
                attendees_confirmed += 1

            fees[attendee.fee]['all'] += 1
            if paid:
                fees[attendee.fee]['paid'] += 1

            if attendee.t_shirt_size:
                size = attendee.t_shirt_size
                tshirts[size]['all'] += 1
                tshirts_total['all'] += 1
                if reconfirm:
                    tshirts[size]['confirmed'] += 1
                    tshirts_total['confirmed'] += 1

            if attendee.shoe_size:
                size = attendee.shoe_size
                shoes[size]['all'] += 1
                shoes_total['all'] += 1
                if reconfirm:
                    shoes[size]['confirmed'] += 1
                    shoes_total['confirmed'] += 1

            attendees_by_country[attendee.country_name]['all'] += 1
            if reconfirm:
                attendees_by_country[attendee.country_name]['confirmed'] += 1

            attendees_by_gender[attendee.gender]['all'] += 1
            if reconfirm:
                attendees_by_gender[attendee.gender]['confirmed'] += 1

            languages = set(
                attendee.languages.lower()
                .replace(',', ' ')
                .replace('/', ' ')
                .replace(';', ' ')
                .split()
            )
            for language in languages:
                attendees_by_language[language]['all'] += 1
                if reconfirm:
                    attendees_by_language[language]['confirmed'] += 1

            try:
                accomm = attendee.accomm
                if not accomm.nights.exists():
                    raise ObjectDoesNotExist
            except ObjectDoesNotExist:
                pass
            else:
                accomm_total += 1
                if reconfirm:
                    accomm_confirmed += 1

                for night in attendee.accomm.nights.all():
                    accommodation[night]['all'] += 1
                    if reconfirm:
                        accommodation[night]['confirmed'] += 1

            try:
                food = attendee.food
                if not food.meals.exists():
                    raise ObjectDoesNotExist
            except ObjectDoesNotExist:
                pass
            else:
                food_total += 1
                if reconfirm:
                    food_confirmed += 1

                markers = []
                if food.diet:
                    markers.append(food.diet.title())
                if not markers:
                    markers = ['No restrictions']

                food_restrictions[' '.join(markers)]['all'] += 1
                if reconfirm:
                    food_restrictions[' '.join(markers)]['confirmed'] += 1

                for meal in food.meals.all():
                    meals[meal.date][meal.meal]['all'] += 1
                    if reconfirm:
                        meals[meal.date][meal.meal]['confirmed'] += 1

        bursaries_by_status = defaultdict(Counter)
        bursaries_travel = Counter()
        for bursary in Bursary.objects.all():
            if not user_is_registered(bursary.user):
                continue
            for type in ('food', 'accommodation', 'travel', 'expenses'):
                if getattr(bursary, 'request_%s' % type):
                    status = getattr(bursary, '%s_status' % type)
                    bursaries_by_status[type]['all'] += 1
                    bursaries_by_status[type][status] += 1
                    if type == 'travel':
                        amount = bursary.travel_bursary
                        bursaries_travel['all'] += amount
                        bursaries_travel[status] += amount

        # Prepare for presentation
        fees = OrderedDict(
            (label, fees[key])
            for key, label in Attendee.FEES.items()
        )
        attendees_by_country = sorted(
            attendees_by_country.items(), key=lambda x: (-x[1]['all'], x[0])
        )
        attendees_by_gender = sorted(
            attendees_by_gender.items(), key=lambda x: (-x[1]['all'], x[0])
        )
        attendees_by_language = sorted(
            attendees_by_language.items(), key=lambda x: (-x[1]['all'], x[0])
        )
        tshirts = OrderedDict(
            (label, tshirts[key])
            for key, label in settings.DEBCONF_T_SHIRT_SIZES
            if key
        )
        shoes = OrderedDict(
            (size, shoes[size])
            for size, label in settings.DEBCONF_SHOE_SIZES
            if size
        )
        if not settings.DEBCONF_SHOE_SIZES:
            shoes = None
        accommodation = OrderedDict(
            sorted(
                (night.date, counts)
                for night, counts in accommodation.items()
            )
        )
        meal_labels = list(Meal.MEALS.values())
        meals = OrderedDict(
            (day, [day_meals[key] for key in Meal.MEALS])
            for day, day_meals in sorted(meals.items())
        )

        bursary_statuses = ['All'] + [
            choice[0].title() for choice in BURSARY_STATUS_CHOICES
        ]

        bursaries_by_status = OrderedDict(
            (type.title(), OrderedDict(
                (status.lower(), counter[status.lower()])
                for status in bursary_statuses
            ))
            for type, counter in bursaries_by_status.items()
        )
        bursaries_travel = OrderedDict(
            (status, bursaries_travel[status.lower()])
            for status in bursary_statuses
        )

        retval = clean_almostdicts({
            'DEBCONF_BURSARY_CURRENCY': settings.DEBCONF_BURSARY_CURRENCY,
            'DEBCONF_REVIEW_FREE_ATTENDEES':
                settings.DEBCONF_REVIEW_FREE_ATTENDEES,
            'attendees_count': attendees_count,
            'attendees_registered': attendees_registered,
            'attendees_approved': attendees_approved,
            'attendees_pending_approval': attendees_pending_approval,
            'attendees_denied_approval': attendees_denied_approval,
            'attendees_confirmed': attendees_confirmed,
            'attendees_arrived': attendees_arrived,
            'fees': fees,
            'tshirts': tshirts,
            'tshirts_total': tshirts_total,
            'shoes': shoes,
            'shoes_total': shoes_total,
            'attendees_by_country': attendees_by_country,
            'attendees_by_language': attendees_by_language,
            'attendees_by_gender': attendees_by_gender,
            'accomm_total': accomm_total,
            'accomm_confirmed': accomm_confirmed,
            'accommodation': accommodation,
            'food_total': food_total,
            'food_confirmed': food_confirmed,
            'food_restrictions': food_restrictions,
            'genders': Attendee.GENDERS,
            'meal_labels': meal_labels,
            'meals': meals,
            'bursary_statuses': bursary_statuses,
            'bursaries_by_status': bursaries_by_status,
            'bursaries_travel': bursaries_travel,
        })

        cache.set(self.cache_key, retval, self.cache_timeout)
        return retval



from django.db import models

from register.fields import RequiredCountries
from register.models.attendee import Attendee


class Visa(models.Model):
    attendee = models.OneToOneField(Attendee, related_name='visa',
                                    on_delete=models.CASCADE)
    country = models.CharField(max_length=2)

    @property
    def country_name(self):
        return RequiredCountries().name(self.country)



# -*- coding: utf-8 -*-

from datetime import date, timedelta

from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from django.template import Context, Template

from django.contrib.sites.models import Site

from bursary.models import Bursary

SUBJECT_TEMPLATE = Template(
    '[{% if final %}FINAL {% endif %}REMINDER] Reimbursement process for '
    '{{ WAFER_CONFERENCE_NAME }} travel bursary recipients')

BODY_TEMPLATE = Template("""\
{% autoescape off %}
Dear {{ object.user.get_full_name }},

According to our records, you still haven't requested your travel bursary
reimbursement for {{ WAFER_CONFERENCE_NAME }}.

{% if final %}
This is the FINAL REMINDER. You have 3 weeks to claim your reimbursement,
before we close the process. The deadline is {{ final_deadline|date }}.

{% endif %}
If you still plan to collect your reimbursement, please do so ASAP. You can
find the instructions in the previous email on the subject.

If you don't intend to collect it, please reply to this e-mail.

If you're having any trouble with the process, please reply to this email and
let us help you.

Thank you
--\u0020
The DebConf Bursaries team
{% endautoescape %}
""")


class Command(BaseCommand):
    help = "Send an email to people who haven't claimed their travel bursary yet"

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually send emails')
        parser.add_argument('--final', action='store_true',
                            help='Give a 3-week final deadline')

    def badger(self, bursary, conference_name, dry_run, final):
        context = {
            'object': bursary,
            'user': bursary.user.username,
            'to': '%s <%s>' % (bursary.user.get_full_name(),
                               bursary.user.email),
            'final': final,
            'final_deadline': date.today() + timedelta(weeks=3),
            'WAFER_CONFERENCE_NAME': conference_name,
        }

        if dry_run:
            print('I would badger {to} (max = {object.travel_bursary})'
                  .format(**context))
            return

        from_email = 'bursaries@debconf.org'
        ctx = Context(context)
        subject = SUBJECT_TEMPLATE.render(ctx)
        body = BODY_TEMPLATE.render(ctx)

        msg = EmailMultiAlternatives(subject, body, to=[context['to']],
                                     from_email=from_email)

        msg.send()

    def handle(self, *args, **options):
        site = Site.objects.get()
        conference_name = site.name.replace(' ', '')
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        to_badger = Bursary.objects.filter(
            request_travel=True,
            travel_status='accepted',
            user__attendee__check_in__pk__isnull=False,
            reimbursed_amount=0,
        ).order_by('user__username')

        for bursary in to_badger:
            self.badger(bursary, conference_name, dry_run, options['final'])



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-07-26 19:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('volunteers', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='volunteer',
            name='tasks',
        ),
        migrations.AlterField(
            model_name='task',
            name='volunteers',
            field=models.ManyToManyField(blank=True, related_name='tasks', to='volunteers.Volunteer'),
        ),
    ]



# Generated by Django 2.2.27 on 2022-07-16 21:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('front_desk', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='checkin',
            name='meal_vouchers',
            field=models.BooleanField(default=False),
        ),
    ]



from django.contrib import admin

from minidebconf.models import Diet, Registration, RegistrationType, ShirtSize


class RegistrationAdmin(admin.ModelAdmin):
    list_display = ('user', 'phone_number', 'registration_type', 'involvement', 'gender', 'country', 'diet', 'shirt_size')
    list_filter = ('registration_type', 'involvement', 'gender', 'days', 'diet', 'shirt_size')

admin.site.register(Registration, RegistrationAdmin)
admin.site.register(RegistrationType)
admin.site.register(Diet)
admin.site.register(ShirtSize)



from django.conf import settings
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import PermissionDenied
from django.views.generic import TemplateView
from django.views.generic.edit import UpdateView, DeleteView
from django.urls import reverse
from minidebconf.forms import register_form_factory
from minidebconf.models import Registration, is_registered


class RegistrationMixin:
    @property
    def user(self):
        return self.request.user

    def get_object(self):
        if is_registered(self.user):
            return Registration.objects.get(user=self.user)
        else:
            return Registration(user=self.user)



class RegisterView(LoginRequiredMixin, RegistrationMixin, UpdateView):
    template_name = 'minidebconf/register.html'

    def dispatch(self, request, *args, **kwargs):
        if not settings.WAFER_REGISTRATION_OPEN:
            raise PermissionDenied("Registration is closed")
        return super().dispatch(request, *args, **kwargs)

    def get_form_class(self):
        return register_form_factory()

    def get_success_url(self):
        return reverse('registration_finished')


class UnregisterView(LoginRequiredMixin, RegistrationMixin, DeleteView):

    def get_success_url(self):
        return reverse('wafer_user_profile', args=[self.user.username])


class RegistrationFinishedView(LoginRequiredMixin, TemplateView):
    template_name = 'minidebconf/registration_finished.html'



from pytest import fixture

from minidebconf import settings as mdc_settings


@fixture(autouse=True)
def minidebconf_settings(settings):
    for k, v in mdc_settings.__dict__.items():
        if k.isupper() and k not in {'DATABASES',}:
            setattr(settings, k, v)


@fixture
def attendee(db, user):
    from minidebconf.models import Registration
    attendee = Registration.objects.create(
        user=user,
        involvement=0,
        gender='n',
        country='CA',
        city_state='BC',
    )
    yield attendee
    attendee.delete()



from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.contrib.sites.models import Site
from django.template import Context, Template
from django.utils.timezone import now

from debconf.tz import aoe_datetime
from register.models import Attendee


SUBJECT = '''Please confirm your {{ WAFER_CONFERENCE_NAME }} attendance'''

TXT = '''Hi {{ name }},

{{ WAFER_CONFERENCE_NAME }} is coming up soon. Please confirm your attendance
so that we can get accurate attendee numbers.

We have you registered as:
Arriving on {{ arrival }}
Departing on {{ departure }}

To confirm your attendance, please go through the registration form once again.
You need to set final arrival and departure dates, select the "My dates are
final" entry and tick the "I confirm my attendance" box.
You need to go through the wizard *until the end* for your confirmation to
be valid! This makes it a perfect time to review your full registration data.

<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/>

{% if before_deadline %}The deadline for confirmation is {{ DEBCONF_CONFIRMATION_DEADLINE|date:"SHORT_DATE_FORMAT" }} AoE
(AoE = Anywhere on Earth: Forget about timezones, just do it by this date)

{% endif %}{% if invoice %}Our records indicate that you haven't settled your invoice for {{ invoice.total }} {{ DEBCONF_BILLING_CURRENCY }}.

While we do accept cash payment on-site, we would strongly prefer if you
settled the invoice directly online, by going to your profile on the DebConf
website. We will not have credit-card facilities available on-site.

<https://{{ WAFER_CONFERENCE_DOMAIN }}/accounts/profile/>

{% endif %}If you no longer plan to attend {{ WAFER_CONFERENCE_NAME }}, go to:
<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/unregister>
and click the "Unregister" button.

See you soon in {{ DEBCONF_CITY }},

The {{ WAFER_CONFERENCE_NAME }} Registration Team
'''


class Command(BaseCommand):
    help = "Badger users who haven't confirmed registration for attendance"

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something')
        parser.add_argument('--site', type=int, default=1,
                            help='Django site ID, default: 1')

    def badger(self, user, dry_run, site):
        attendee = user.attendee
        invoice = attendee.new_invoices.first()
        confirmation_deadline = settings.DEBCONF_CONFIRMATION_DEADLINE
        before_deadline = aoe_datetime(confirmation_deadline) > now()
        context = Context({
            'name': user.userprofile.display_name(),
            'arrival': attendee.arrival,
            'before_deadline': before_deadline,
            'departure': attendee.departure,
            'invoice': invoice,
            'DEBCONF_BILLING_CURRENCY': settings.DEBCONF_BILLING_CURRENCY,
            'DEBCONF_CITY': settings.DEBCONF_CITY,
            'DEBCONF_CONFIRMATION_DEADLINE': confirmation_deadline,
            'WAFER_CONFERENCE_DOMAIN': site.domain,
            'WAFER_CONFERENCE_NAME': site.name,
        })

        txt = Template(TXT).render(context)
        subject = Template(SUBJECT).render(context)
        to = user.email
        if dry_run:
            print('I would badger:', to)
            return
        email_message = EmailMultiAlternatives(subject, txt, to=[to])
        email_message.send()

    def handle(self, *args, **options):
        if not settings.RECONFIRMATION:
            print('Set RECONFIRMATION=True and try again')
            return

        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')
        site = Site.objects.get(id=1)
        for attendee in Attendee.objects.filter(reconfirm=False):
            user = attendee.user
            if user.userprofile.is_registered():
                self.badger(user, dry_run, site)



# Generated by Django 2.2.15 on 2020-08-18 22:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('volunteers', '0004_volunteer_timezone'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='tasktemplate',
            options={'permissions': (('accept_video_tasks', 'Can accept video tasks'),)},
        ),
        migrations.AddField(
            model_name='task',
            name='required_permission',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='tasktemplate',
            name='required_permission',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



from django.urls import re_path

from bursary.views import (
    AttendanceRequestReview,
    BursaryMassUpdate, BursaryRefereeAdd, BursaryRefereeExport,
    BursaryRequestExport, BursaryUpdate,
)

urlpatterns = [
    re_path(r'^admin/export/requests/$', BursaryRequestExport.as_view(),
        name='bursaries_admin_export_requests'),
    re_path(r'^admin/export/referees/$', BursaryRefereeExport.as_view(),
        name='bursaries_admin_export_referees'),
    re_path(r'^admin/add_referees/$', BursaryRefereeAdd.as_view(),
        name='bursaries_admin_add_referees'),
    re_path(r'^admin/update_requests/$', BursaryMassUpdate.as_view(),
        name='bursaries_admin_update_requests'),
    re_path(r'^admin/attendee-review/$', AttendanceRequestReview.as_view(),
        name='bursaries_admin_review_attendance'),
    re_path(r'^$', BursaryUpdate.as_view(), name='bursary_update'),
]



# Generated by Django 4.2.8 on 2023-12-23 17:48

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("minidebconf", "0004_alter_registration_phone_number"),
    ]

    operations = [
        migrations.AddField(
            model_name="registration",
            name="arranged_accommodation",
            field=models.BooleanField(
                default=False, verbose_name="Conference-arranged accommodation"
            ),
        ),
        migrations.AddField(
            model_name="registration",
            name="arranged_food",
            field=models.BooleanField(
                default=False, verbose_name="Conference-arranged food"
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from django.contrib.auth import get_user_model

from invoices.prices import invoice_user


class Command(BaseCommand):
    help = 'Create an invoice for a user'

    def add_arguments(self, parser):
        parser.add_argument('username', help='The user to invoice')
        parser.add_argument('--dry-run', action='store_true',
            help="Print the invoice we would create, without creating it.")

    def handle(self, *args, **options):
        username = options['username']
        dry_run = options['dry_run']
        User = get_user_model()
        user = User.objects.get(username=username)
        invoice = invoice_user(user, save=not dry_run)
        print(invoice)



import datetime

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.utils import timezone

from invoices.models import Invoice, InvoiceLine
from register.dates import get_ranges_for_dates

DEBCONF_NAME = settings.DEBCONF_NAME
INVOICE_PREFIX = settings.INVOICE_PREFIX
PRICES = settings.PRICES


def meal_prices():
    """Return a dict of meal name: price
    And the currency the prices are specified in.
    """
    prices = {name: meal['price']
              for name, meal in settings.PRICES['meal'].items()}
    prices['currency'] = settings.DEBCONF_BILLING_CURRENCY
    return prices


def meal_price_string():
    """Return a human-readable string of daily meal prices"""
    prices = meal_prices()
    s = [f'{meal.title()} {prices[meal]} {settings.DEBCONF_BILLING_CURRENCY}'
         for meal in ('breakfast', 'lunch', 'dinner')
         if meal in prices]
    return ', '.join(s) + '.'


def invoice_user(user, force=False, save=False):
    from bursary.models import Bursary

    attendee = user.attendee

    try:
        bursary = user.bursary
    except Bursary.DoesNotExist:
        bursary = Bursary()

    lines = []
    fee = PRICES['fee'][attendee.fee]
    if fee['price']:
        lines.append(InvoiceLine(
            reference='{}REG-{}'.format(INVOICE_PREFIX, attendee.fee.upper()),
            description='{} {} registration fee'.format(
                DEBCONF_NAME, fee['name']),
            unit_price=fee['price'],
            quantity=1,
        ))

    try:
        accomm = attendee.accomm
    except ObjectDoesNotExist:
        accomm = None

    if accomm and not bursary.potential_bursary('accommodation'):
        for line in invoice_accomm(accomm):
            lines.append(InvoiceLine(**line))

    try:
        food = attendee.food
    except ObjectDoesNotExist:
        food = None

    if food and not bursary.potential_bursary('food'):
        for line in invoice_food(food, accomm):
            lines.append(InvoiceLine(**line))

    for paid_invoice in user.invoices.filter(status='paid', compound=False):
        lines.append(InvoiceLine(
            reference='INV#{}'.format(paid_invoice.reference_number),
            description='Previous Payment Received',
            unit_price=-paid_invoice.total,
            quantity=1,
        ))

    invoice = Invoice(
        recipient=user,
        status='new',
        date=timezone.now(),
        invoiced_entity=attendee.invoiced_entity,
        billing_address=attendee.billing_address
    )

    total = sum(line.total for line in lines)

    # Only save invoices if non-zero
    if save and total > 0:
        invoice.save()

    for i, line in enumerate(lines):
        line.line_order = i
        if save:
            line.invoice_id = invoice.id
            line.save()

    return {
        'invoice': invoice,
        'lines': lines,
        'total': total,
        'total_local': total * settings.DEBCONF_LOCAL_CURRENCY_RATE,
    }


def invoice_food(food, accomm=None):
    """Generate one invoice line per meal type per consecutive stay"""
    from register.models.food import Meal

    accomm_included_meals = set()
    if accomm:
        accom_price = PRICES['accomm'][accomm.option]
        accomm_included_meals = accom_price.get('included_meals', set())

    for meal, meal_label in Meal.MEALS.items():
        dates = [entry.date for entry in food.meals.filter(meal=meal)
                 if not entry.conference_dinner]
        if not dates:
            continue
        if meal in accomm_included_meals:
            continue

        ranges = get_ranges_for_dates(dates)
        for first, last in ranges:
            n_meals = (last - first).days + 1

            if first != last:
                dates = '{} to {}'.format(first, last)
            else:
                dates = str(first)

            yield {
                'reference': '{}{}'.format(INVOICE_PREFIX, meal.upper()),
                'description': '{} {} ({})'.format(
                    DEBCONF_NAME, meal_label, dates),
                'unit_price': PRICES['meal'][meal]['price'],
                'quantity': n_meals,
            }

    if food.meals.filter(meal='dinner',
                         date=settings.DEBCONF_CONFERENCE_DINNER_DAY):
        food_price = PRICES['meal']['conference_dinner']
        yield {
            'reference': '{}CONFDINNER'.format(INVOICE_PREFIX),
            'description': '{} {} ({})'.format(
                DEBCONF_NAME,
                food_price.get('name', 'Conference Dinner'),
                settings.DEBCONF_CONFERENCE_DINNER_DAY.isoformat()),
            'unit_price': food_price['price'],
            'quantity': 1,
        }


def invoice_accomm(accomm):
    """Generate one invoice line per consecutive stay"""
    stays = get_ranges_for_dates(
        night.date for night in accomm.nights.all()
    )
    accom_price = PRICES['accomm'][accomm.option]

    if accom_price.get('paid_separately', False):
        return
    if 'price' not in accom_price:
        return

    for first_night, last_night in stays:
        last_morning = last_night + datetime.timedelta(days=1)
        num_nights = (last_morning - first_night).days
        dates = "evening of %s to morning of %s" % (first_night,
                                                    last_morning)
        yield {
            'reference': f'{INVOICE_PREFIX}ACCOMM-{accomm.option.upper()}',
            'description':
                f'{DEBCONF_NAME} {accom_price["description"]} ({dates})',
            'unit_price': accom_price['price'],
            'quantity': num_nights,
        }



from collections import defaultdict

from django.utils.timezone import now

from volunteers.models import Volunteer
from register.models import Attendee


issued = {
 'Cathylafourmi',
 'KristiProgri',
 'PotatoFormula',
 'aeb',
 'ajfus',
 'andi',
 'arthurmde',
 'cate',
 'cherubin',
 'helen.fornazier',
 'hpdang',
 'jcc',
 'jmw',
 'kanashiro',
 'kobla',
 'kroeckx',
 'lfilipoz',
 'mariobehling',
 'medicalwei-guest',
 'nattie',
 'olasd',
 'onovy',
 'owlfox',
 'philh',
 'phls-guest',
 'piotr',
 'pollo',
 'sheldon_knuth',
 'siqueira',
 'stefanor',
 'taffit',
 'tobi',
 'tzafrir',
 'urbec',
 'valessio-guest',
 'whalebox4.5',
 'zobel',
 'zumbi',
}

def new_shirts():
    volunteers = []
    by_size = defaultdict(list)

    for volunteer in Volunteer.objects.select_related(
                'user',
                'user__attendee',
                'user__userprofile',
            ).all():
        user = volunteer.user
        try:
            attendee = user.attendee
        except Attendee.DoesNotExist:
            continue

        v = {
            'username': user.username,
            'name': user.userprofile.display_name(),
            'shirt_size': attendee.t_shirt_size,
            'tasks': 0
        }
        for task in volunteer.tasks.all():
            template = task.template
            if not template or not template.video_task:
                continue
            if task.start > now():
                continue
            v['tasks'] += 1

        if v['tasks']:
            volunteers.append(v)
        if v['tasks'] >= 3 and v['username'] not in issued:
            by_size[v['shirt_size']].append(v['username'])

    volunteers.sort(key=lambda v: v['tasks'], reverse=True)
    print(by_size)



from django.conf import settings

from register.forms.personal import PersonalInformationForm
from register.models.attendee import Attendee
from register.models.address import Address
from register.models.queue import Queue
from register.views.core import RegisterStep


class PersonalInformationView(RegisterStep):
    title = 'Personal Information'
    template_name = 'register/page/personal.html'
    form_class = PersonalInformationForm

    def get_initial(self):
        user = self.request.user
        initial = {}

        attendee = user.attendee
        for field in attendee._meta.get_fields():
            if field.is_relation:
                continue
            initial[field.name] = getattr(attendee, field.name)

        shipping_address = attendee.shipping_address
        if shipping_address:
            for field in shipping_address._meta.get_fields():
                if field.is_relation:
                    continue
                initial[f'shipping_{field.name}'] = getattr(
                    shipping_address, field.name)

        return initial

    def form_valid(self, form):
        user = self.request.user
        data = form.cleaned_data

        user.attendee  # We should never be creating, here
        user.attendee = Attendee.objects.update_or_create(
            user=user, defaults=data)[0]

        if form.cleaned_data['t_shirt_size'] or form.cleaned_data['shoe_size']:
            if settings.DEBCONF_ONLINE:
                Address.objects.update_or_create(
                    attendee=user.attendee, role='shipping',
                    defaults={
                        field[9:]: value
                        for field, value in data.items()
                        if field.startswith('shipping_')
                    })
        else:
            Address.objects.filter(
                attendee=user.attendee, role='shipping').delete()

        return super().form_valid(form)

    def registration_completed(self, attendee):
        if attendee.t_shirt_size or attendee.shoe_size:
            queue = Queue.objects.get_or_create(name='Swag')[0]
            queue.slots.get_or_create(attendee=attendee)
        if attendee.pgp_fingerprints and settings.ISSUE_KSP_ID:
            queue = Queue.objects.get_or_create(name='PGP Keysigning')[0]
            queue.slots.get_or_create(attendee=attendee)



import re

from django.conf import settings
from django import forms
from django.core.exceptions import ValidationError

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Field, Fieldset, Layout
from django_countries.fields import LazyTypedChoiceField

from register.fields import OptionalCountries, RequiredCountries
from register.models.attendee import Attendee


T_SHIRT_CHART_LINK = '<a href="/about/t-shirts">t-shirt sizes chart</a>'
SHOE_CHART_LINK = '<a href="/about/shoe-sizes">shoe size chart</a>'


class PersonalInformationForm(forms.Form):
    t_shirt_size = forms.ChoiceField(
        label='My t-shirt size',
        choices=settings.DEBCONF_T_SHIRT_SIZES,
        help_text='Refer to the ' + T_SHIRT_CHART_LINK + '.',
        required=False,
    )
    shoe_size = forms.ChoiceField(
        label='My shoe size',
        choices=settings.DEBCONF_SHOE_SIZES,
        help_text='Refer to the ' + SHOE_CHART_LINK + '.',
        required=False,
    )
    shipping_contact = forms.CharField(
        label='Recipient Name',
        help_text='Required. Addressee for the package',
        max_length=50,
        required=False,
    )
    shipping_phone_number = forms.CharField(
        label='Phone Number',
        help_text='Optional, but strongly recommended. '
                  'Useful for customs and couriers to contact you.',
        max_length=32,
        required=False,
    )
    shipping_company = forms.CharField(
        label='Company Name',
        help_text='Optional, use if shipping to a company.',
        max_length=50,
        required=False,
    )
    shipping_line1 = forms.CharField(
        label='Address Line 1',
        help_text='Required. Usually the street name and number.',
        max_length=50,
        required=False,
    )
    shipping_line2 = forms.CharField(
        label='Address Line 2',
        help_text='Optional, usually the district / suburb',
        max_length=50,
        required=False,
    )
    shipping_city = forms.CharField(
        label='City/Town',
        max_length=50,
        required=False,
        help_text='Required',
    )
    shipping_province = forms.CharField(
        label='State/Province',
        max_length=50,
        required=False,
        help_text='Required',
    )
    shipping_postal_code = forms.CharField(
        label='Postal/ZIP Code',
        max_length=32,
        required=False,
    )
    shipping_country = LazyTypedChoiceField(
        label='Country',
        choices=RequiredCountries(),
        required=False,
        help_text='Required',
    )
    gender = forms.ChoiceField(
        label='My gender',
        choices=Attendee.GENDERS.items(),
        help_text='For diversity statistics.',
        required=False,
    )
    country = LazyTypedChoiceField(
        label='The country I call home',
        help_text='For diversity statistics.',
        choices=OptionalCountries(),
        required=False,
    )
    languages = forms.CharField(
        label='The languages I speak',
        help_text='We will list these on your name-tag.',
        initial='en',
        max_length=50,
        required=False,
    )
    pgp_fingerprints = forms.CharField(
        label='My PGP key fingerprints for keysigning',
        help_text='Optional. One fingerprint per line, if you want to take '
                  'part in <a href="https://wiki.debconf.org/wiki/DebConf18/'
                  'Keysigning">the continuous keysigning party</a>. '
                  'Will appear on nametags and a keysigning notesheet, with '
                  'no verification done by the conference organisers.<br>'
                  'Just provide the fingerprints, e.g. <code>1234 5678 90AB '
                  'CDEF 0000  1111 2222 3333 4444 5555</code>',
        widget=forms.Textarea(attrs={'rows': 2}),
        required=False,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.include_media = False
        if settings.DEBCONF_SHOE_SIZES:
            shoe_size = (Field('shoe_size'),)
        else:
            shoe_size = ()
        if settings.DEBCONF_ONLINE:
            shipping_address = (
                Fieldset(
                    'Shipping Address for swag (e.g. t-shirts)',
                    'shipping_contact',
                    'shipping_phone_number',
                    'shipping_company',
                    'shipping_line1',
                    'shipping_line2',
                    'shipping_city',
                    'shipping_province',
                    'shipping_postal_code',
                    'shipping_country',
                    css_id='shipping_address',
                ),
            )
        else:
            shipping_address = ()
        self.helper.layout = Layout(
            Fieldset(
                'Demographic Information',
                Field('gender'),
                Field('country'),
            ),
            Fieldset(
                'Swag',
                Field('t_shirt_size', id='t_shirt_size'),
                *shoe_size,
            ),
            *shipping_address,
        )
        if not settings.DEBCONF_ONLINE:
            self.helper.layout.append('languages'),
            self.helper.layout.append('pgp_fingerprints'),

    def clean(self):
        cleaned_data = super().clean()
        if settings.DEBCONF_ONLINE and (
                cleaned_data.get('t_shirt_size')
                or cleaned_data.get('shoe_size')):
            for field in (
                        'shipping_contact',
                        'shipping_line1',
                        'shipping_city',
                        'shipping_province',
                        'shipping_country',
                    ):
                if not cleaned_data.get(field):
                    self.add_error(field, 'Required.')

    def clean_pgp_fingerprints(self):
        fingerprints = self.cleaned_data.get('pgp_fingerprints').strip()
        if not fingerprints:
            return ''
        if '-----BEGIN' in fingerprints:
            raise ValidationError('Only fingerprints, not keys, please')
        cleaned = []
        for line in fingerprints.splitlines():
            fp = line.strip().upper().replace(' ', '')
            if fp.startswith('0x'):
                fp = fp[2:]
            if not re.match(r'^[0-9A-F]{40}$', fp):
                raise ValidationError(
                    '{} is not a PGP fingerprint'.format(line))
            cleaned.append('{} {} {} {} {}  {} {} {} {} {}'.format(
                *[fp[i:i + 4] for i in range(0, 40, 4)]))
        return '\n'.join(cleaned)



import datetime

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import permission_required
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.models import Group
from django.core.cache import cache
from django.db.models import F, Q, Sum
from django.http import HttpResponseForbidden
from django.shortcuts import redirect
from django.urls import reverse, reverse_lazy
from django.utils.dateparse import parse_duration
from django.utils.decorators import method_decorator
from django.views.generic import FormView, DetailView
from django.views.generic.edit import UpdateView
from django.views.generic.list import ListView

from bakery.views import BuildableTemplateView
from wafer.kv.models import KeyValue
from wafer.users.views import EditOneselfMixin

from volunteers.forms import (
    TasksFilterForm, VideoMassCreateTaskForm, VideoShirtForm,
    VolunteerUpdateForm)
from volunteers.models import Task, TaskTemplate, Volunteer


class TasksView(LoginRequiredMixin, ListView):
    model = Task
    template_name = 'wafer.volunteers/tasks.html'

    queryset = Task.objects.annotate_all()

    def get_context_data(self, **kwargs):
        context = super(TasksView, self).get_context_data(**kwargs)

        # TODO Find a better way
        if self.request.user.is_authenticated:
            volunteer, created = Volunteer.objects.get_or_create(
                user=self.request.user)

        if self.request.GET:
            filter_form = TasksFilterForm(self.request.GET,
                                          timezone=volunteer.timezone)
        else:
            filter_form = TasksFilterForm(timezone=volunteer.timezone)

        if filter_form.is_valid():
            filter_data = filter_form.cleaned_data
        else:
            filter_data = filter_form.initial

        context['filter_form'] = filter_form

        tasks = context['object_list'].filter(
            start__lte=filter_data['end'],
            end__gte=filter_data['start'],
        )

        if 'needed' in filter_data['extra_filters']:
            tasks = tasks.filter(
                nbr_volunteers__lt=F('max_volunteers'),
            )

        context['tasks'] = tasks

        if (self.request.user.is_authenticated and
                'preferred' in filter_data['extra_filters']):
            try:
                volunteer = Volunteer.objects.get(user=self.request.user)
                # TODO: add support for the preferred_tasks as well
                context['tasks'] = (
                    context['tasks'].filter(
                        Q(category__in=volunteer.preferred_categories.all())
                        | Q(template__in=volunteer.preferred_task_types.all())
                    )
                )
            except Volunteer.DoesNotExist:
                pass

        return context


class TaskView(LoginRequiredMixin, DetailView):
    model = Task
    template_name = 'wafer.volunteers/task.html'

    queryset = Task.objects.annotate_all()

    def get_context_data(self, **kwargs):
        context = super(TaskView, self).get_context_data(**kwargs)
        # TODO Find a better way
        context['already_volunteer'] = (
            self.request.user.is_authenticated and
            self.object.volunteers.filter(user=self.request.user).exists()
        )
        context['can_volunteer'] = (
            self.request.user.is_authenticated and
            self.object.open_for_signup
        )

        context['concurrent_tasks'] = self.object.concurrent_tasks()

        return context

    def post(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            return HttpResponseForbidden()
        self.object = self.get_object()
        volunteer, new = Volunteer.objects.get_or_create(user=request.user)
        concurrently_volunteering = self.object.concurrent_tasks().filter(
            volunteers=volunteer).all()

        required_permission = self.object.get_required_permission()

        if self.object in volunteer.tasks.all():
            self.object.volunteers.remove(volunteer)
            messages.info(request, 'Removed')
        elif concurrently_volunteering:
            messages.error(request, (
                'You are already volunteering in this time-slot. {}. '
                'You can only be in one place at a time.'
            ).format(concurrently_volunteering[0]))
        elif self.object.nbr_volunteers >= self.object.max_volunteers:
            messages.error(request, 'This task already has enough volunteers.')
        elif not self.object.open_for_signup:
            messages.error(request,
                'This task has already ended. '
                'It is too late to volunteer for it.')
        elif required_permission and not request.user.has_perm(
                required_permission):
            messages.error(request,
                'Sorry, but this task requires volunteers to be trained first. '
                'Please talk to the the appropriate team, and they can train '
                'you. Thanks for helping out!')
        else:
            self.object.volunteers.add(volunteer)
            messages.success(request, 'Thank you for volunteering!')

        return redirect('wafer_task', pk=self.object.pk)


class VolunteerView(EditOneselfMixin, DetailView):
    model = Volunteer
    slug_field = 'user__username'
    template_name = 'wafer.volunteers/volunteer.html'

    def get_object(self, *args, **kwargs):
        # TODO Find a better way
        if self.request.user.is_authenticated:
            Volunteer.objects.get_or_create(user=self.request.user)

        return super(VolunteerView, self).get_object(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(VolunteerView, self).get_context_data(**kwargs)
        context['profile'] = self.object.user.userprofile
        return context


class VolunteerUpdate(EditOneselfMixin, UpdateView):
    model = Volunteer
    slug_field = 'user__username'
    form_class = VolunteerUpdateForm
    template_name = 'wafer.volunteers/volunteer_update.html'

    def get_object(self, *args, **kwargs):
        # TODO Find a better way
        if self.request.user.is_authenticated:
            Volunteer.objects.get_or_create(user=self.request.user)

        return super(VolunteerUpdate, self).get_object(*args, **kwargs)

    def get_success_url(self):
        return reverse('wafer_volunteer',
                       kwargs={'slug': self.object.user.username})


class VideoMassScheduleView(LoginRequiredMixin, FormView):
    @method_decorator(permission_required('volunteers.add_task'))
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['lead_time'] = self.lead_time
        return kwargs

    @property
    def lead_time(self):
        lead_time = self.request.GET.get('lead_time')
        if lead_time:
            lead_time = parse_duration(lead_time)
        return lead_time or parse_duration('P1D')

    form_class = VideoMassCreateTaskForm
    template_name = 'wafer.volunteers/admin/video_mass_schedule.html'
    success_url = reverse_lazy('wafer_tasks')


class VideoShirtView(LoginRequiredMixin, FormView):
    form_class = VideoShirtForm
    template_name = 'wafer.volunteers/admin/video_shirts.html'
    success_url = reverse_lazy('wafer_tasks')

    @method_decorator(permission_required('volunteers.add_task'))
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        issued = []
        video_admin_group = Group.objects.get_by_natural_key('Volunteer Admin')
        for kv in KeyValue.objects.filter(
                group=video_admin_group, key='video_shirt',
                userprofile__isnull=False):
            userprofile = kv.userprofile_set.first()
            issued.append('{}: {}'.format(userprofile.user.username,
                                          userprofile.display_name()))
        context['issued'] = sorted(issued)

        return context


class VolunteerStatisticsView(BuildableTemplateView):
    build_path = 'volunteers/statistics/index.html'
    template_name = 'wafer.volunteers/statistics.html'
    cache_key = 'volunteers:statistics'
    cache_timeout = 30*60 if not settings.DEBUG else 10

    def get_context_data(self, **kwargs):
        retval = cache.get(self.cache_key)
        if retval:
            return retval

        volunteers_by_template = {}
        for template in TaskTemplate.objects.all():
            volunteers = Volunteer.objects.filter(tasks__template=template)
            volunteers_by_template[template.name] = {
                'unique_volunteers': volunteers.distinct().count(),
                'volunteers': volunteers.count(),
            }

        volunteers = []
        for volunteer in Volunteer.objects.all():
            if not volunteer.tasks.exists():
                continue
            total_task_data = (
                volunteer.tasks
                .annotate_all()
                .annotate(duration=F('end') - F('start'))
                .values('name_', 'duration')
                .annotate(total=Sum('duration'))
                .order_by('-total')
            )
            fav_task_data = total_task_data[0]
            total_duration = sum((type['total'] for type in total_task_data),
                                 datetime.timedelta())
            volunteers.append({
                'name': volunteer.user.userprofile.display_name(),
                'tasks': total_duration,
                'favourite_task': fav_task_data['name_'],
                'favourite_task_count': fav_task_data['total'],
            })

        volunteers.sort(key=lambda v: v['tasks'], reverse=True)

        retval = {
            'volunteers': volunteers,
            'volunteers_by_template': volunteers_by_template,
        }

        cache.set(self.cache_key, retval, self.cache_timeout)
        return retval



import datetime
from django.conf import settings
from django.utils import timezone
from wafer.schedule.models import ScheduleBlock

def expose_settings(request):
    return {
        'TIME_ZONE': settings.TIME_ZONE,
        'DEBCONF_ONLINE': settings.DEBCONF_ONLINE,
        'WAFER_TALKS_OPEN': settings.WAFER_TALKS_OPEN,
        'SITE_DESCRIPTION': settings.SITE_DESCRIPTION,
        'SITE_AUTHOR': settings.SITE_AUTHOR,
        'USING_BADGES_APP': "badges" in settings.INSTALLED_APPS,
        'USING_FRONTDESK_APP': "front_desk" in settings.INSTALLED_APPS,
        'USING_REGISTER_APP': "register" in settings.INSTALLED_APPS,
        'USING_VOLUNTEERS_APP': "volunteers" in settings.INSTALLED_APPS,
    }


def is_it_debconf(request):
    today = timezone.now().date()
    two_days = datetime.timedelta(days=2)
    first = ScheduleBlock.objects.order_by('start_time').first()
    last = ScheduleBlock.objects.order_by('end_time').last()
    if not first or not last:
        return {}

    start = first.start_time.date()
    end =  last.end_time.date()
    return {
        'debconf_day': start <= today <= end,
        'debconf_soon': start - two_days <= today < start,
    }



from datetime import datetime, timedelta

from django.utils.timezone import get_fixed_timezone, make_aware

AoE = get_fixed_timezone(timedelta(hours=-12))


def aoe_datetime(deadline_date):
    """Given a date() object, return a timezone-aware datetime() object,
    representing the end of day, anywhere on earth (AoE).
    """
    dt = datetime(deadline_date.year, deadline_date.month, deadline_date.day)
    dt += timedelta(days=1)  # midnight at the end of day
    return make_aware(dt, AoE)



# Generated by Django 2.2.27 on 2022-03-16 01:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0012_covid19'),
    ]

    operations = [
        migrations.CreateModel(
            name='Visa',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('country', models.CharField(max_length=2)),
                ('attendee', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='visa', to='register.Attendee')),
            ],
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0005_renumber_queue_monotonic_position'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='queueslot',
            unique_together=set([('queue', 'monotonic_position')]),
        ),
    ]



from django.core.management.base import BaseCommand

from register.models.attendee import Attendee
from register.models.queue import Queue


class Command(BaseCommand):
    help = ('Flush the PGP Keysigning queue and re-create, in sorted order. '
            'This lets us create a mostly-sorted list, at a freeze point.')

    def handle(self, *args, **options):
        queue = Queue.objects.get_or_create(name='PGP Keysigning')[0]
        queue.slots.all().delete()
        players = Attendee.objects.exclude(pgp_fingerprints='')
        players = players.order_by( 'user__first_name', 'user__last_name')
        for attendee in players:
            queue.slots.get_or_create(attendee=attendee)



# Generated by Django 4.2.8 on 2023-12-23 20:34

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("minidebconf", "0006_registration_notes_registration_travel_cost_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="registration",
            name="diet",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="minidebconf.diet",
                verbose_name="Diet",
            ),
        ),
    ]



from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.contrib.sites.models import Site
from django.template import Context, Template

from register.models import Attendee


SUBJECT = ('[ACTION REQUIRED]: Are you coming to {{ WAFER_CONFERENCE_NAME }}? '
           'Confirm your attendance *now*!')

TXT = """\
Dear {{ name }},

Our records indicate that you haven't confirmed your attendance at {{ WAFER_CONFERENCE_NAME }}.

The deadline for confirmation is {{ DEBCONF_CONFIRMATION_DEADLINE|date:"SHORT_DATE_FORMAT" }} AoE.
(AoE = Anywhere on Earth: Forget about timezones, just do it by this date)
After this date, we cannot guarantee any accommodation or food that you may
have requested. Your badge may not be printed and we probably won't have a
t-shirt or swag to offer you, either.

To confirm your attendance, go through the registration form once again.
You need to set final arrival and departure dates, select the "My dates are
final" entry and tick the "I confirm my attendance" box. Then go through the
rest of the form verifying your data and Save.

<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/>

If you no longer plan to attend {{ WAFER_CONFERENCE_NAME }}, go to:
<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/unregister>
and click the "Unregister" button.

Thank you,

The {{ WAFER_CONFERENCE_NAME }} Registration Team
"""


class Command(BaseCommand):
    help = "Final badger for users who haven't confirmed registration for attendance"

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something')
        parser.add_argument('--site', type=int, default=1,
                            help='Django site ID, default: 1')

    def badger(self, user, dry_run, site):
        confirmation_deadline = settings.DEBCONF_CONFIRMATION_DEADLINE
        context = Context({
            'name': user.userprofile.display_name(),
            'DEBCONF_CONFIRMATION_DEADLINE': confirmation_deadline,
            'WAFER_CONFERENCE_DOMAIN': site.domain,
            'WAFER_CONFERENCE_NAME': site.name,
        })

        txt = Template(TXT).render(context)
        subject = Template(SUBJECT).render(context)
        to = user.email
        if dry_run:
            print('I would badger:', to)
            return
        email_message = EmailMultiAlternatives(subject, txt, to=[to])
        email_message.send()

    def handle(self, *args, **options):
        if not settings.RECONFIRMATION:
            print('Set RECONFIRMATION=True and try again')
            return

        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')
        site = Site.objects.get(id=1)
        for attendee in Attendee.objects.filter(reconfirm=False):
            user = attendee.user
            if user.userprofile.is_registered():
                self.badger(user, dry_run, site)



import csv

from django.core.management.base import BaseCommand

from dc17.models import Attendee, Food


class Command(BaseCommand):
    help = 'Output a CSV dump of nametag data'

    def handle(self, *args, **options):
        w = csv.writer(self.stdout)
        w.writerow(
            ('Name', 'Line 2', 'Line 3', 'Languages', 'Diet'))
        for attendee in Attendee.objects.all():
            try:
                diet = attendee.food.diet
            except Food.DoesNotExist:
                diet = ''

            w.writerow((
                attendee.user.userprofile.display_name(),
                attendee.nametag_2,
                attendee.nametag_3,
                attendee.languages,
                diet,
            ))



from collections import namedtuple
import pytest

from django.conf import settings

from debconf.templatetags.debconf import hls_stream_url
from debconf.templatetags.debconf import rtmp_stream_url
from debconf.templatetags.debconf import irc_channels


Venue = namedtuple("Venue", ["name"])


@pytest.fixture
def venue():
    return Venue(name="Room 1")


class TestHLSStreamURL:
    def test_fixed_url(self, monkeypatch, venue):
        monkeypatch.setattr(
            settings, "DEBCONF_VENUE_STREAM_HLS_URL", "https://foo.bar/stream.m3u8"
        )
        assert hls_stream_url(venue) == "https://foo.bar/stream.m3u8"

    def test_url_by_venue_name(self, monkeypatch, venue):
        monkeypatch.setattr(
            settings, "DEBCONF_VENUE_STREAM_HLS_URL", "https://foo.bar/{name}.m3u8"
        )
        assert hls_stream_url(venue) == "https://foo.bar/room-1.m3u8"


class TestRTMPStreamURL:
    def test_fixed_url(self, monkeypatch, venue):
        monkeypatch.setattr(
            settings, "DEBCONF_VENUE_STREAM_RTMP_URL", "rtmp://foo.bar/stream"
        )
        assert rtmp_stream_url(venue, "src") == "rtmp://foo.bar/stream"

    def test_url_by_venue_name(self, monkeypatch, venue):
        monkeypatch.setattr(
            settings,
            "DEBCONF_VENUE_STREAM_RTMP_URL",
            "rtmp://foo.bar/stream/{name}_{quality}",
        )
        assert rtmp_stream_url(venue, "src") == "rtmp://foo.bar/stream/room-1_src"


class TestIRCChannels:
    def test_fixed_channel(self, monkeypatch, venue):
        monkeypatch.setattr(settings, "DEBCONF_VENUE_IRC_CHANNELS", ["#debconf"])
        assert irc_channels(venue) == ["#debconf"]

    def test_room_channel_plus_general_channel(self, monkeypatch, venue):
        monkeypatch.setattr(
            settings, "DEBCONF_VENUE_IRC_CHANNELS", ["#myconf-{name}", "#myconf"]
        )
        assert irc_channels(venue) == ["#myconf-room-1", "#myconf"]



import os
from django.core.management.base import BaseCommand
from django.utils import translation
from django.utils.translation import gettext_lazy as _

from wafer.pages.models import Page


PAGES = [
    {
        "name": _("Home page"),
        "slug": "index",  # can't be translated
        "include_in_menu": False,
    },
    {
        "name": _("About"),
        "slug": _("about"),
        "include_in_menu": True,
        "children": [
            {
                "name": _("About the event"),
                "slug": _("event"),
                "include_in_menu": True,
            },
            {
                "name": _("Code of Conduct"),
                "slug": _("coc"),
                "include_in_menu": True,
            },
            {
                "name": _("Organizers"),
                "slug": _("org"),
                "include_in_menu": True,
            },
        ]
    },
    {
        "name": _("Contribute"),
        "slug": _("contribute"),
        "include_in_menu": True,
        "children": [
            {
                "name": _("Call for Proposals"),
                "slug": _("cfp"),
                "include_in_menu": True,
            },
            {
                "name": _("Important dates"),
                "slug": _("important-dates"),
                "include_in_menu": True,
            },
        ]
    },
    {
        "name": _("Schedule"),
        "slug": "schedule",  # can't be translated
        "include_in_menu": True,
    },
]


class Command(BaseCommand):
    help = 'Create sample set of pages for a MiniDebConf'

    def handle(self, *args, **options):
        if Page.objects.count():
            return
        self.create_pages(None, PAGES)

    def create_pages(self, parent, pages):
        for i, entry in enumerate(pages):
            entry["name"] = str(entry["name"])
            entry["slug"] = str(entry["slug"])
            self.create_page(menu_order=i, parent=parent, **entry)

    def create_page(self, **data):
        try:
            children = data.pop("children")
        except KeyError:
            children = []
        name = data["name"]
        content = f"# {name}\n\n"
        page = Page.objects.create(content=content, **data)
        print(f"Created page {page}")
        if children:
            self.create_pages(page, children)




from django.conf.urls import include
from django.urls import re_path
from minidebconf.views import RegisterView, UnregisterView, RegistrationFinishedView

urlpatterns = [
    re_path(r'^register/$', RegisterView.as_view(), name='register'),
    re_path(r'^unregister/$', UnregisterView.as_view(), name='unregister'),
    re_path(r'^register/finished/$', RegistrationFinishedView.as_view(), name='registration_finished'),
    re_path(r'', include('debconf.urls')),
    re_path(r'', include('wafer.urls')),
]
